'use strict';
var _ = require('underscore');
var React = require('react/addons');
var Modal = require('react-modal');
var I18N = require('jsxlate/lib/components');
var ReactIntl = require('react-intl');
var App = require('dealers/App');
var Client = require('dealers/Client');
var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');
var Analytics = require('elements/Analytics');
var Centered = require('elements/Centered');
var HtmlHead = require('elements/HtmlHead');
var Navigation = require('Navigation');
var PageFooter = require('elements/PageWrapper/PageFooter');
var PageWrapper = require('elements/PageWrapper/PageWrapper');
var ResizeWatcher = require('elements/ResizeWatcher.jsx');
var Router = require('Router');
var SignupFlow = require('elements/LoginSignup/SignupFlow');
var Worldly = require('Worldly');
var ClickWrapper = React.createClass({
    mixins: [
        Worldly,
        Navigation
    ],
    propTypes: { additionalClickHandler: React.PropTypes.func },
    getDefaultProps() {
        return {
            additionalClickHandler: () => {
            }
        };
    },
    onClick(event) {
        this.navigateWithClick(event);
        this.props.additionalClickHandler.call(this, event);
    },
    route: function (event) {
        if (event.target.tagName == 'A') {
            return Router.routeForUrlPath(event.target.pathname);
        }
    },
    render() {
        return (
            <div onClick={this.onClick}>
            {this.props.children}
        </div>
        );
    }
});
var Layout = React.createClass({
    mixins: [ReactIntl.IntlMixin],
    componentDidMount: function () {
        this.world().onActionPerformed(this.forceUpdate.bind(this));
        this.updateBodyClasses();
    },
    componentWillUpdate() {
        this.updateBodyClasses();
    },
    childContextTypes: {
        IDO_i5_world_do_not_use_this: React.PropTypes.object.isRequired,
        locales: React.PropTypes.arrayOf(React.PropTypes.string)
    },
    getChildContext() {
        return {
            IDO_i5_world_do_not_use_this: this.props.world,
            locales: [this.props.world.get('Locale')]
        };
    },
    world() {
        return this.props.world;
    },
    hideMobileNavMenuIfShown(event) {
        var mobileNavMenu = document.getElementById('mobile-user-menu');
        if (mobileNavMenu && mobileNavMenu.contains(event.target)) {
            return;
        }
        if (NavMenu.isMobileNavMenuOpen(this.world())) {
            event.preventDefault();
            NavMenu.toggleMobileNavMenu(this.world());
        }
    },
    render: function () {
        return (
            <ClickWrapper additionalClickHandler={this.hideMobileNavMenuIfShown}>
            <ResizeWatcher onResize={this.onResize} />
            <Modal
                    isOpen={LoginSignup.isModalSignupOpen(this.world())}
                    closeTimeoutMS={300}
                    onRequestClose={event => event.key === 'Escape' && this.closeSignup()}
                    contentWillFocus={this.onModalContentWillFocus}>
                {}
                <ClickWrapper>
                    <Centered>
                        <SignupFlow ref="flow" onClose={this.closeSignup} />
                    </Centered>
                    <a className="reactmodal-close" title="close" href="#"
                            onClick={this.closeSignup}></a>
                </ClickWrapper>
            </Modal>
            <PageWrapper>
                <HtmlHead info={this.pageComponentType().htmlHead(this.world())} />
                {this.pageComponent()}
            </PageWrapper>
            <PageFooter />
            <Analytics />
        </ClickWrapper>
        );
    },
    onModalContentWillFocus: function () {
        if (document.addEventListener) {
            document.addEventListener('keydown', this.closeSignupListener);
        } else {
            window.attachEvent('onkeydown', this.closeSignupListener);
        }
        var flow = this.refs.flow;
        if (flow && flow.isMounted()) {
            flow.focus();
        }
    },
    closeSignup() {
        if (document.removeEventListener) {
            document.removeEventListener('keydown', this.closeSignupListener);
        } else {
            window.detachEvent('onkeydown', this.closeSignupListener);
        }
        LoginSignup.closeSignup(this.world());
    },
    closeSignupListener(event) {
        if (event.keyCode === 27) {
            this.closeSignup();
        }
    },
    onResize: function (dims) {
        Client.setDimensions(this.world(), dims);
        setTimeout(this.updateBodyClasses, 1);
    },
    updateBodyClasses() {
        var classes = {
            mobile: Client.isMobile(this.world()),
            'nav-menu': NavMenu.isMobileNavMenuOpen(this.world()),
            'modal-open': LoginSignup.isModalSignupOpen(this.world())
        };
        var bodyClasses = _.extend(classes, this.pageWrapperFeatures());
        _.each(bodyClasses, (value, key) => {
            if (value) {
                this.addClass(document.body, key);
            } else {
                this.removeClass(document.body, key);
            }
        });
    },
    addClass(el, className) {
        if (el.classList) {
            el.classList.add(className);
        } else {
            el.className += ' ' + className;
        }
    },
    removeClass(el, className) {
        if (el.classList) {
            el.classList.remove(className);
        } else {
            el.className = el.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
        }
    },
    pageWrapperFeatures() {
        var pageSpecificFeatures = this.pageComponentType().pageWrapperFeatures;
        return _.extend({}, this.defaultPageWrapperFeatures, pageSpecificFeatures);
    },
    defaultPageWrapperFeatures: {
        'has-mobile-subheader': true,
        'has-sub-header': true,
        'has-search': true
    },
    pageComponent: function () {
        return React.createElement(this.pageComponentType(), {});
    },
    pageComponentType: function () {
        var route = Router.routeForWorld(this.world());
        return Router.componentTypeForRoute(route);
    }
});
module.exports = Layout;
