var React = require('react');
var guid = 0;
var k = function () {
};
var eq = function (a, b) {
    return a === b;
};
var addClass = require('./add-class');
var ComboboxOption = require('./option');
function getLabel(component) {
    var hasLabel = component.props.label != null;
    return hasLabel ? component.props.label : component.props.children;
}
function matchFragment(userInput, firstChildLabel) {
    userInput = userInput.toLowerCase();
    firstChildLabel = firstChildLabel.toLowerCase();
    if (userInput === '' || userInput === firstChildLabel) {
        return false;
    }
    if (firstChildLabel.toLowerCase().indexOf(userInput.toLowerCase()) === -1) {
        return false;
    }
    return true;
}
module.exports = React.createClass({
    propTypes: {
        autocomplete: React.PropTypes.oneOf([
            'both',
            'inline',
            'list'
        ]),
        onInput: React.PropTypes.func,
        onInputKeyUp: React.PropTypes.func,
        onFocusOption: React.PropTypes.func,
        onBlur: React.PropTypes.func,
        onSelect: React.PropTypes.func,
        valueComparator: React.PropTypes.func,
        value: React.PropTypes.any,
        tabIndex: React.PropTypes.string,
        maxMenuHeight: React.PropTypes.number
    },
    getDefaultProps: function () {
        return {
            autocomplete: 'both',
            onInput: k,
            onFocusOption: k,
            onBlur: k,
            onSelect: k,
            valueComparator: eq,
            value: null,
            maxMenuHeight: Infinity
        };
    },
    getInitialState: function () {
        return {
            value: this.props.value,
            inputValue: this.findInputValue(),
            isOpen: false,
            focusedIndex: null,
            matchedAutocompleteOption: null,
            usingKeyboard: false,
            activedescendant: null,
            menu: {
                children: [],
                activedescendant: null,
                isEmpty: true
            }
        };
    },
    componentWillMount: function () {
        this.setState({ menu: this.makeMenu() });
    },
    componentWillReceiveProps: function (newProps) {
        var newState = { menu: this.makeMenu(newProps.children) };
        if (newProps.value && newProps.value !== this.props.value) {
            newState.inputValue = this.findInputValue(newProps.value);
        }
        this.setState(newState);
    },
    handleButtonClick: function () {
        this.state.isOpen ? this.hideList() : this.showList();
        this.focusInput();
    },
    handleKeydown: function (event) {
        var handlerName = this.inputKeydownMap[event.keyCode];
        if (!handlerName) {
            return;
        }
        if (this.state.isOpen) {
            event.preventDefault();
        } else if (event.keyCode === 9) {
            return;
        }
        this.setState({ usingKeyboard: true });
        this[handlerName].call(this, event);
    },
    handleOptionKeyDown: function (child, event) {
        var handlerName = this.optionKeydownMap[event.keyCode];
        if (!handlerName) {
            if (event.keyCode !== 9) {
                this.selectInput();
            } else {
                event.preventDefault();
            }
            return;
        }
        event.preventDefault();
        this.setState({ usingKeyboard: true });
        this[handlerName].call(this, child);
    },
    handleOptionMouseEnter: function (index) {
        if (this.state.usingKeyboard) {
            this.setState({ usingKeyboard: false });
        } else {
            this.focusOptionAtIndex(index);
        }
    },
    handleInputKeyUp: function (event) {
        if (this.props.onInputKeyUp && this.props.onInputKeyUp(event) === false) {
            return;
        }
        if (this.state.menu.isEmpty || event.keyCode === 8 || !this.props.autocomplete.match(/both|inline/)) {
            return;
        }
        this.autocompleteInputValue();
    },
    handleInputChange: function () {
        var value = this._input.value;
        this.clearSelectedState(function () {
            this.props.onInput(value);
            if (!this.state.isOpen) {
                this.showList();
            }
        }.bind(this), { inputValue: value });
    },
    handleInputBlur: function (event) {
        var focusedAnOption = this.state.focusedIndex != null;
        if (focusedAnOption) {
            return;
        }
        this.maybeSelectAutocompletedOption();
        this.hideList(event);
    },
    inputKeydownMap: {
        9: 'selectFirstOption',
        13: 'selectOnEnter',
        27: 'hideOnEscape',
        38: 'focusPrevious',
        40: 'focusNext'
    },
    optionKeydownMap: {
        13: 'selectOptionAndClose',
        27: 'hideOnEscape',
        38: 'focusPrevious',
        40: 'focusNext'
    },
    handleOptionBlur: function (event) {
        this.blurTimer = setTimeout(function () {
            this.hideList(event);
        }.bind(this), 0);
    },
    handleOptionFocus: function () {
        clearTimeout(this.blurTimer);
    },
    menuEvents: [
        'resize',
        'scroll',
        'orientationchange'
    ],
    showList: function () {
        if (this.props.autocomplete.match(/both|list/)) {
            this.setState({ isOpen: true });
        }
        this.addMenuEventHandlers();
    },
    hideList: function (event) {
        if (this.isMounted()) {
            this.setState({ isOpen: false });
        }
        this.removeMenuEventHandlers();
        this.props.onBlur(event);
    },
    addMenuEventHandlers: function () {
        this.menuEvents.forEach(eventName => {
            if (window.addEventListener) {
                window.addEventListener(eventName, this.handleMenuEvent);
            } else {
                window.attachEvent('on' + eventName, this.handleMenuEvent);
            }
        });
    },
    removeMenuEventHandlers: function () {
        this.menuEvents.forEach(eventName => {
            if (window.removeEventListener) {
                window.removeEventListener(eventName, this.handleMenuEvent);
            } else {
                window.detachEvent('on' + eventName, this.handleMenuEvent);
            }
        });
    },
    handleMenuEvent: function (evt) {
        this.forceUpdate();
    },
    componentWillUnmount: function () {
        this.removeMenuEventHandlers();
    },
    hideOnEscape: function (event) {
        this.hideList(event);
        this.focusInput();
    },
    focusInput: function () {
        this._input.focus();
    },
    selectInput: function () {
        this._input.select();
    },
    selectOnEnter: function () {
        this.maybeSelectAutocompletedOption();
        this.selectInput();
    },
    maybeSelectAutocompletedOption: function () {
        if (!this.state.matchedAutocompleteOption) {
            return;
        }
        this.selectOption(this.state.matchedAutocompleteOption, { focus: false });
    },
    makeMenu: function (children) {
        var activedescendant;
        var isEmpty = true;
        children = children || this.props.children;
        var newChildren = React.Children.map(children, function (child, index) {
            if (child === null || child.type !== ComboboxOption) {
                return child;
            }
            isEmpty = false;
            var valueMatch = this.props.valueComparator(this.state.value, child.props.value);
            var newId = valueMatch ? child.props.id || 'rf-combobox-selected-' + ++guid : child.props.id;
            if (valueMatch) {
                activedescendant = newId;
            }
            return React.cloneElement(child, {
                id: newId,
                isSelected: valueMatch,
                onBlur: function (evt) {
                    this.handleOptionBlur(evt);
                }.bind(this),
                onClick: this.selectOption.bind(this, child, { hide: true }),
                onTouchEnd: function () {
                    if (!this.cancelSelect) {
                        this.selectOption(child, { hide: true });
                    }
                }.bind(this),
                onFocus: this.handleOptionFocus,
                onKeyDown: evt => this.handleOptionKeyDown(child, evt),
                onMouseEnter: this.handleOptionMouseEnter.bind(this, index)
            }, child.props.children);
        }.bind(this));
        return {
            children: newChildren,
            activedescendant: activedescendant,
            isEmpty: isEmpty
        };
    },
    getClassName: function () {
        var className = addClass(this.props.className, 'rf-combobox');
        if (this.state.isOpen) {
            className = addClass(className, 'rf-combobox-is-open');
        }
        return className;
    },
    clearSelectedState: function (cb, overrides) {
        var newState = Object.assign({}, {
            focusedIndex: null,
            inputValue: null,
            value: null,
            matchedAutocompleteOption: null,
            activedescendant: null
        }, overrides);
        this.setState(newState, cb);
    },
    autocompleteInputValue: function () {
        if (this.props.autocomplete == false || this.props.children.length === 0) {
            return;
        }
        var input = this._input;
        var inputValue = input.value;
        var firstChild = this.props.children.length ? this.props.children[0] : this.props.children;
        var label = getLabel(firstChild);
        var fragment = matchFragment(inputValue, label);
        if (!fragment) {
            return;
        }
        input.value = label;
        input.setSelectionRange(inputValue.length, label.length);
        this.setState({ matchedAutocompleteOption: firstChild });
    },
    selectOption: function (child, options) {
        options = options || {};
        this.setState({
            value: child.props.value,
            inputValue: getLabel(child),
            matchedAutocompleteOption: null
        }, function () {
            this.props.onFocusOption(child.props.value, child);
            if (options.hide === true) {
                this.props.onSelect(child.props.value, child);
                this.hideList();
            }
            if (options.focus === true) {
                this.selectInput();
            }
        }.bind(this));
    },
    selectOptionAndClose: function (child) {
        this.selectOption(child, {
            hide: true,
            focus: true
        });
    },
    selectFirstOption: function (event) {
        if (this.state.isOpen && !event.nativeEvent.shiftKey) {
            event.preventDefault();
            this.focusOptionAtIndex(0, true);
            setTimeout(function () {
                if (this.isMounted()) {
                    this.focusInput();
                }
            }.bind(this), 1);
        }
    },
    focusNext: function (event) {
        if (this.state.menu.isEmpty) {
            return;
        }
        var index = this.state.focusedIndex == null ? 0 : this.state.focusedIndex + 1;
        this.focusOptionAtIndex(index);
    },
    focusPrevious: function (event) {
        if (this.state.menu.isEmpty) {
            return;
        }
        var last = this.props.children.length - 1;
        var index = this.state.focusedIndex == null ? last : this.state.focusedIndex - 1;
        this.focusOptionAtIndex(index);
    },
    focusSelectedOption: function () {
        var selectedIndex;
        React.Children.forEach(this.props.children, function (child, index) {
            if (this.props.valueComparator(child.props.value, this.state.value)) {
                selectedIndex = index;
            }
        }.bind(this));
        this.showList();
        this.setState({ focusedIndex: selectedIndex }, this.focusOption);
    },
    findInputValue: function (value) {
        value = value || this.props.value;
        var inputValue;
        React.Children.forEach(this.props.children, function (child) {
            if (this.props.valueComparator(child.props.value, value)) {
                inputValue = getLabel(child);
            }
        }.bind(this));
        return inputValue || value;
    },
    focusOptionAtIndex: function (index, andHide) {
        if (!this.state.isOpen && this.state.value) {
            return this.focusSelectedOption();
        }
        this.showList();
        var length = React.Children.count(this.props.children);
        if (index === -1) {
            index = length - 1;
        } else if (index === length) {
            index = 0;
        }
        this.setState({ focusedIndex: index }, this.focusOption);
        var focusedChild;
        React.Children.forEach(this.props.children, function (child, childIndex) {
            if (childIndex === index) {
                focusedChild = child;
            }
        });
        this.selectOption(focusedChild, {
            focus: false,
            hide: andHide
        });
    },
    focusOption: function () {
        var index = this.state.focusedIndex || 0;
        var node = React.findDOMNode(this.refs.list).childNodes[index];
        node.focus();
    },
    recordBoundingBox: function () {
        var node = this._input;
        this._rect = node.getBoundingClientRect();
    },
    componentDidMount: function () {
        this.recordBoundingBox();
    },
    componentDidUpdate: function () {
        this.recordBoundingBox();
    },
    portalStyles: function () {
        var input = this._input;
        if (!input) {
            return {};
        } else {
            var windowHeight;
            if (window.innerHeight) {
                windowHeight = window.innerHeight;
            } else {
                windowHeight = document.documentElement.clientHeight;
            }
            return {
                position: 'absolute',
                display: this.state.isOpen ? 'block' : 'none',
                overflow: 'auto',
                maxHeight: this._rect ? Math.min(this.props.maxMenuHeight, windowHeight - this._rect.bottom) : 0
            };
        }
    },
    render: function () {
        return (
            <div className={this.getClassName()}>
                <input type="text"
                    ref={elm => this._input = React.findDOMNode(elm)}
                    placeholder={this.props.placeholder}
                    disabled={this.props.disabled} className="rf-combobox-input"
                    tabIndex={this.props.tabIndex}
                    defaultValue={this.props.value}
                    value={this.state.inputValue}
                    onChange={this.handleInputChange}
                    onBlur={this.handleInputBlur}
                    onKeyDown={this.handleKeydown}
                    onKeyUp={this.handleInputKeyUp}
                    role="combobox"
                    aria-activedescendant={this.state.menu.activedescendant}
                    aria-autocomplete={this.props.autocomplete}
                    aria-owns="react-autocomplete-results" />
                <span aria-hidden="true" className="rf-combobox-button" onClick={this.handleButtonClick}>▾</span>
                <div style={{ position: 'relative' }}>
                    {React.Children.count(this.props.children) > 0 && (
                        <div
                            style={this.portalStyles()} id="react-autocomplete-results" ref="list"
                            className="rf-combobox-list"
                            aria-expanded={this.state.isOpen + ''}
                            onTouchStart={evt => {
                                this.cancelSelect = false;
                            }}
                            onTouchMove={evt => {
                                this.cancelSelect = true;
                            }} role="listbox">{this.state.menu.children}
                    </div>
                    )}
                </div>
            </div>
        );
    }
});
