'use strict';
var React = require('react');
var _ = require('underscore');
var moment = require('moment');
var {I18N, i18n} = require('I18N');
var BoxWidget = require('elements/BoxWidget');
var ContentGroup = require('elements/ContentGroup');
var If = require('abc/if');
var LinkedMap = require('elements/LinkedMap');
var GradFairDetails = React.createClass({
    statics: {
        accessibilities: {
            'guide_dogs': (
                <li key="guide_dogs"><I18N
                        message={'Guide dogs are welcome'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Guide dogs are welcome</span>;
                        }} /></li>
            ),
            'wheelchair_access': (
                <li key="wheelchair_access"><I18N
                        message={'There is wheelchair access'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>There is wheelchair access</span>;
                        }} /></li>
            )
        }
    },
    render: function () {
        var doc = this.props.fair;
        var address = doc.addresses[0] || { city: '' };
        return (
            <BoxWidget headline="Details">
            <ContentGroup headline="Date and time">
                <p><I18N
                            message={'Fair starts:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Fair starts:</span>;
                            }} /> {moment(doc.utc_start).format('MMMM Do, YYYY, hh:mm a')}</p>
                <p><I18N
                            message={'Fair ends:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Fair ends:</span>;
                            }} /> {moment(doc.utc_end).format('MMMM Do, YYYY, hh:mm a')}</p>
            </ContentGroup>

            <ContentGroup headline="Location">
                <p>{doc.location}</p>
                    <LinkedMap location={address} width={228} height={228} />
            </ContentGroup>

            {this.props.children}

            <If test={doc.access_other}>
                <ContentGroup headline="Accessibility">
                    <p><I18N
                                message={'Statement of inclusion:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Statement of inclusion:</span>;
                                }} /> {doc.access_other}</p>
                <If test={doc.access}>
                    <I18N
                                message={'In this venue:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>In this venue:</span>;
                                }} />
                    <ul>{_.map(doc.access, this.labelForAccessibility)}</ul>
                </If>
                </ContentGroup>
            </If>
            <If test={doc.fair_email}>
                <ContentGroup headline="Contact">
                    <p><I18N
                                message={'Got questions? Email us at: {doc.fair_email}'}
                                context={this}
                                args={[doc]}
                                fallback={function () {
                                    return <span>Got questions? Email us at: {doc.fair_email}</span>;
                                }} /></p>
                </ContentGroup>
            </If>
        </BoxWidget>
        );
    },
    labelForAccessibility: function (token) {
        return GradFairDetails.accessibilities[token];
    }
});
module.exports = GradFairDetails;
