'use strict';
var _ = require('underscore');
var cuid = require('cuid');
var Immutable = require('immutable');
var React = require('react');
var Rx = require('rx');
var superagent = require('superagent');
var Autocomplete = require('elements/Autocomplete');
var API = require('API');
var Geo = require('dealers/Geo');
var Router = require('Router');
var Wish = require('Wish');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var Field = require('elements/Forms/Field');
var InputWrapper = require('elements/Forms/InputWrapper');
var ValidationMessage = require('elements/Forms/ValidationMessage');
var TrapdoorSelect = require('elements/TrapdoorSelect');
var Validation = require('elements/Validation');
var type = React.PropTypes;
var LocationField = React.createClass({
    propTypes: {
        value: type.object.isRequired,
        onChange: type.func.isRequired,
        onSelect: type.func,
        onKeyUp: type.func,
        onFocus: type.func,
        onBlur: type.func,
        validation: type.object,
        element: type.object,
        className: type.string,
        allowFallback: type.bool,
        tabIndex: type.string,
        maxMenuHeight: type.number
    },
    getDefaultProps: function () {
        return {
            allowFallback: true,
            className: '',
            autocompleteEndpointOptions: {}
        };
    },
    mixins: [Worldly],
    statics: {
        blankLocation: function (overrides) {
            return Immutable.Map({
                fallback: false,
                countryGeoid: undefined,
                regionGeoid: undefined,
                countryCode: undefined,
                regionCode: undefined,
                countryName: undefined,
                regionName: undefined,
                city: undefined,
                location: undefined,
                geoid: undefined
            }).merge(overrides);
        }
    },
    getInitialState: function () {
        return { currentSearchPrefix: '' };
    },
    fallbackPrompt: function () {
        return (
            <p className="powertip-like location-not-found" style={{ marginTop: '1.5rem' }}>
            <I18N
                    message={'Hmm... we couldn\'t find that place in our database.\n                Are you sure it\'s correct?'}
                    context={this}
                    args={[]}
                    fallback={function () {
                        return (
                            <span>Hmm... we couldn't find that place in our database.
                Are you sure it's correct?</span>
                        );
                    }} />
            <span className="actions">
                <a href="#" ref="yesFallback" onClick={this.yesFallback}><I18N
                            message={'Yes'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Yes</span>;
                            }} /></a>
                <a href="#" ref="noFallback" onClick={this.noFallback}><I18N
                            message={'No'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>No</span>;
                            }} /></a>
            </span>
        </p>
        );
    },
    yesFallback: function (event) {
        event.preventDefault();
        var bestGuessCity = this.state.currentSearchPrefix.split(',')[0];
        var location = LocationField.blankLocation({
            fallback: true,
            city: bestGuessCity
        });
        this.props.onSelect && this.props.onSelect(location);
        this.props.onChange && this.props.onChange(location);
    },
    noFallback: function (event) {
        event.preventDefault();
        this.setSearchInput('');
        setTimeout(() => this.refs.source.focusInput(), 1);
        this.props.onChange(LocationField.blankLocation());
    },
    fallbackWasReset: function (event) {
        this.noFallback(event);
    },
    fetchLocations: function (prefix, options, cb) {
        var locale = this.world().get('Locale');
        if (prefix === '') {
            cb({});
        } else {
            Geo.autocompleteResults(prefix, Object.assign({ lang: locale }, options), cb);
        }
    },
    setupSignals: function () {
        var NotQueried = {};
        function signalFromCallbackWithOptions(cb, options) {
            return function (inputSignal) {
                return inputSignal.flatMapLatest(inputValue => Rx.Observable.fromCallback(cb)(inputValue, options));
            };
        }
        function signalOfAThatEmitsWhenAOrBChanges(As, Bs) {
            return Rx.Observable.combineLatest(As, Bs, (a, b) => a);
        }
        this.prefixes = new Rx.Subject();
        var interestingPrefixes = this.prefixes.map(prefix => prefix.length >= 3 ? prefix : '');
        var resultsFn = signalFromCallbackWithOptions(this.fetchLocations, this.props.autocompleteEndpointOptions);
        var results = resultsFn(interestingPrefixes).share();
        var receivedEmptyResults = results.map(result => result !== NotQueried && result.length === 0);
        var resultsAreEmptyWithPrefixChanges = signalOfAThatEmitsWhenAOrBChanges(receivedEmptyResults, interestingPrefixes);
        var locationIsUnrecognized = Rx.Observable.combineLatest(resultsAreEmptyWithPrefixChanges, interestingPrefixes, (resultsAreEmpty, prefix) => resultsAreEmpty && prefix.length > 0).distinctUntilChanged();
        results.map(result => result === NotQueried ? [] : result).subscribe(this.setTypeaheadResults);
        locationIsUnrecognized.subscribe(this.setLocationIsUnrecognized);
    },
    setLocationIsUnrecognized: function (isUnrecognized) {
        this.world().performAction({
            name: 'setLocationIsUnrecognized',
            transaction: function (worldState) {
                return worldState.setIn([
                    'Geo',
                    'locationIsUnrecognized',
                    this._guid
                ], isUnrecognized);
            }.bind(this)
        });
    },
    setTypeaheadResults: function (results) {
        this.world().performAction({
            name: 'setTypeaheadResults',
            transaction: function (worldState) {
                return worldState.setIn([
                    'Geo',
                    'autocompleteResults',
                    this._guid
                ], Immutable.fromJS(results));
            }.bind(this)
        });
        if (results.length === 0) {
            this.props.onChange(LocationField.blankLocation());
        }
    },
    componentWillUnmount() {
        var previousRootNodeId = this._guid;
        this.world().performAction({
            name: 'clearLocationState',
            transaction: ws => ws.deleteIn([
                'Geo',
                'autocompleteResults',
                previousRootNodeId
            ]).deleteIn([
                'Geo',
                'locationIsUnrecognized',
                previousRootNodeId
            ])
        });
    },
    typeaheadOptions: function () {
        return this.world().getIn([
            'Geo',
            'autocompleteResults',
            this._guid
        ], Immutable.List());
    },
    locationIsUnrecognized: function () {
        return this.world().getIn([
            'Geo',
            'locationIsUnrecognized',
            this._guid
        ], false);
    },
    render: function () {
        return (
            <div
                className={'field location-editor ' + this.props.className}
                onKeyUp={this.props.onKeyUp}
                onFocus={this.props.onFocus}
                onBlur={this.props.onBlur}>
            {this.props.value.get('fallback') ? this.renderFallbackMode() : this.renderTypeaheadMode()}
        </div>
        );
    },
    renderTypeaheadMode() {
        var typeaheadOpts = this.typeaheadOptions().map(place => (
            <Autocomplete.Option key={place.get('geoid')} value={place.get('matched_name')}>
                            {place.get('matched_name')}
                       </Autocomplete.Option>
        )).toJS();
        var location = this.props.value.get('location');
        return (
            <div>
            <Autocomplete.Combobox ref='source'
                    placeholder={this.props.placeholder}
                    value={location}
                    autocomplete='list'
                    tabIndex={this.props.tabIndex}
                    onInput={this.setSearchInput}
                    onBlur={this.props.onBlur}
                    maxMenuHeight={this.props.maxMenuHeight}
                    onFocusOption={(value, element) => this.pickLocation(value)}
                    onSelect={this.handleTypeaheadSelect}>
                {typeaheadOpts}
            </Autocomplete.Combobox>
            {this.props.validation && <Validation validation={this.props.validation} position={this.position} />}
            {this.props.showErrors && <ValidationMessage topPosition={37} messages={this.props.element.members.geoid.errors} />}
            {this.locationIsUnrecognized() && this.props.allowFallback && this.fallbackPrompt()}
        </div>
        );
    },
    handleTypeaheadSelect(value) {
        this.pickLocation(value, this.props.onSelect);
    },
    renderFallbackMode() {
        return (
            <LocationWidgetFallback
                value={this.props.value}
                validation={this.props.validation}
                onReset={this.fallbackWasReset}
                onChange={value => {
                    this.props.onChange(value);
                    this.props.onSelect && this.props.onSelect(value);
                }}
                element={this.props.element}
                showErrors={this.props.showErrors} />
        );
    },
    onFocus: function (event) {
        if (!this.getDOMNode().contains(event.relatedTarget)) {
            this.props.onFocus(event);
        }
    },
    onBlur: function (event) {
        if (!this.getDOMNode().contains(event.relatedTarget)) {
            this.props.onBlur(event);
        }
    },
    setSearchInput: function (location) {
        let callback = this.props.onSelect || this.props.onChange;
        callback(LocationField.blankLocation({ location }));
        this.prefixes.onNext(location);
    },
    pickLocation: function (name, callback) {
        callback = callback || this.props.onChange;
        var loc = this.typeaheadOptions().filter(o => o.get('matched_name') == name).first();
        callback(LocationField.blankLocation(Immutable.Map({
            geoid: loc.get('geoid'),
            location: loc.get('matched_name'),
            countryCode: loc.get('country'),
            regionCode: loc.get('code') || undefined,
            type: loc.get('type')
        })));
    },
    componentDidMount: function () {
        this._guid = cuid();
        this.setupSignals();
        this.prefixes.subscribe(prefix => this.setState({ currentSearchPrefix: prefix }));
    },
    componentWillUpdate: function () {
        this.position = this.getDOMNode().offsetTop;
    }
});
var GeoLocationList = React.createClass({
    render() {
        return (
            <TrapdoorSelect onChange={this.onChange} value={this.props.value}>
            {this.props.options.toJS().map(pair => <option key={pair.geoid} value={pair.geoid}>{pair.name}</option>)}
        </TrapdoorSelect>
        );
    },
    onChange(event) {
        this.props.onChange(parseInt(event.target.value, 0));
    }
});
var CountryList = React.createClass({
    mixins: [Worldly],
    propTypes: {
        value: React.PropTypes.number,
        onChange: React.PropTypes.func.isRequired
    },
    getInitialState: function () {
        return { countries: Immutable.List() };
    },
    componentDidMount: function () {
        Geo.getCountries(this.world().get('Locale'), result => this.setState({ countries: result }));
    },
    render: function () {
        return <GeoLocationList value={this.props.value} options={this.state.countries} onChange={this.onChange} />;
    },
    onChange: function (geoid) {
        var selected = this.state.countries.filter(val => val.geoid === geoid);
        this.props.onChange(selected.first());
    }
});
var RegionList = React.createClass({
    mixins: [Worldly],
    propTypes: {
        value: React.PropTypes.oneOfType([
            React.PropTypes.number,
            React.PropTypes.object
        ]),
        onChange: React.PropTypes.func.isRequired
    },
    handleOptions: function (list) {
        if (this.props.value !== Geo.NotApplicable && list.size === 0) {
            _.defer(() => this.props.onChange(Geo.NotApplicable));
        }
        return list;
    },
    render: function () {
        var options = this.props.countryGeoid === undefined ? Immutable.List() : Wish.look(Geo.getRegionsForCountry(this.world(), this.props.countryGeoid), {
            Pending: () => Immutable.List(),
            Error: () => Immutable.List(),
            Available: this.handleOptions
        });
        var value = this.props.value === Geo.NotApplicable ? undefined : this.props.value;
        return <GeoLocationList value={value} options={options} onChange={this.onChange} />;
    },
    onChange: function (geoid) {
        var regionsWish = Geo.readRegionsForCountry(this.world(), this.props.countryGeoid);
        if (regionsWish.get('state') !== 'Available') {
            return;
        }
        var selected = regionsWish.get('value').filter(val => val.geoid === geoid);
        this.props.onChange(selected.first());
    }
});
var SubField = React.createClass({
    mixins: [Field],
    propTypes: {
        label: React.PropTypes.oneOfType([
            React.PropTypes.string,
            React.PropTypes.element
        ]),
        type: React.PropTypes.oneOfType([
            React.PropTypes.string,
            React.PropTypes.func
        ]),
        className: React.PropTypes.string,
        showErrors: React.PropTypes.bool
    },
    getInitialState() {
        return {
            errorRight: 30,
            errorTop: 45
        };
    },
    render() {
        let element = this.schemaElement();
        let Input = typeof this.props.type === 'string' ? InputWrapper : this.props.type;
        return (
            <div style={{ position: 'relative' }}>
            <label>{this.props.label}</label>
            <Input {...this.props} ref="input"
                    value={element.value}
                    onChange={this.onChange} />
            {this.props.validation && <Validation validation={this.props.validation.children.country} leftPosition={30} />}
            {this.props.showErrors && <ValidationMessage topPosition={this.state.errorTop} rightPosition={this.state.errorRight} messages={this.props.element.errors} />}
        </div>
        );
    },
    componentDidMount() {
        let fullBox = React.findDOMNode(this).getBoundingClientRect();
        let inputBox = React.findDOMNode(this.refs.input).getBoundingClientRect();
        let errorRight = fullBox.width - inputBox.width;
        let errorTop = inputBox.height;
        this.setState({
            errorRight,
            errorTop
        });
    }
});
var LocationWidgetFallback = React.createClass({
    render: function () {
        return (
            <div className="fallback-form">
            <fieldset>
                <SubField
                        type={CountryList}
                        label={(
                            <I18N
                                message={'Country:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Country:</span>;
                                }} />
                        )}
                        onChange={this.countryChanged}
                        showErrors={this.props.showErrors} ref="country"
                        element={this.props.element.members.countryGeoid} />
                <SubField
                        type={RegionList}
                        label={(
                            <I18N
                                message={'Region:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Region:</span>;
                                }} />
                        )}
                        onChange={this.regionChanged}
                        countryGeoid={this.props.value.get('countryGeoid')}
                        showErrors={this.props.showErrors} ref="region"
                        element={this.props.element.members.regionGeoid} />
                <SubField type="text"
                        label={(
                            <I18N
                                message={'City:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>City:</span>;
                                }} />
                        )}
                        onChange={this.cityChanged}
                        value={this.props.value.get('city')}
                        showErrors={this.props.showErrors}
                        element={this.props.element.members.city} />
                <a href="#" style={{ float: 'right' }} onClick={this.props.onReset}>
                    <I18N
                            message={'Enter a different location'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Enter a different location</span>;
                            }} />
                </a>
            </fieldset>
        </div>
        );
    },
    updateValue: function (update) {
        this.props.onChange(this.props.value.merge(update));
    },
    cityChanged: function (city) {
        this.updateValue({ city });
    },
    countryChanged: function (country) {
        this.updateValue({
            countryGeoid: country.geoid,
            regionGeoid: undefined,
            countryCode: country.code,
            regionCode: undefined,
            countryName: country.name,
            regionName: undefined
        });
    },
    regionChanged: function (region) {
        if (Immutable.is(region, Geo.NotApplicable)) {
            this.updateValue({
                regionGeoid: Geo.NotApplicable,
                regionCode: '',
                regionName: ''
            });
        } else {
            this.updateValue({
                regionGeoid: region.geoid,
                regionCode: region.code,
                regionName: region.name
            });
        }
    }
});
module.exports = LocationField;
