'use strict';
var React = require('react');
var Immutable = require('immutable');
var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var Valid = require('abc/validators');
var Wish = require('Wish');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var Engage = require('./JanrainEngage');
var FormInput = require('elements/FormInput');
var PaneNavButtons = require('elements/PaneFlow').PaneNavButtons;
var Spinner = require('elements/Spinner');
var ForgotPassword = require('./ForgotPassword');
var LoginPassword = React.createClass({
    mixins: [
        Worldly,
        Engage.Mixin
    ],
    getInitialState() {
        return { showPasswordValidation: false };
    },
    componentDidUpdate: function () {
        if (!Auth.isUnauthed(this.world())) {
            this.props.onClose();
        }
    },
    login: function () {
        this.setState({ showPasswordValidation: true });
        Auth.logInByEmail(this.world(), Signin.getField('email'), Signin.getField('password'));
    },
    retry: function () {
        Auth.retryLogIn(this.world(), Signin.getField('email'), Signin.getField('password'));
    },
    render: function () {
        var email = Signin.getField('email');
        var password = Signin.getField('password');
        var attempt = Auth.loginAttemptWithEmailAndPassword(this.world(), email, password);
        var attemptStatus;
        var failureMessage;
        if (attempt) {
            attemptStatus = Wish.look(attempt, {
                Available: v => {
                    let error = v.get('error');
                    if (error) {
                        if (v.getIn([
                                'body',
                                'error'
                            ]) === 'Pending verification.') {
                            return 'UnverifiedEmail';
                        } else {
                            return 'BadPassword';
                        }
                    }
                    return 'Success';
                },
                Pending: () => <Spinner />,
                Error: () => 'Error'
            });
        } else {
            attemptStatus = 'NotAttempted';
        }
        if (attemptStatus === 'UnverifiedEmail') {
            setTimeout(() => this.props.onUnverified(), 0);
        }
        var attemptMessage = {
            'Success': i18n('You have logged in.'),
            'UnverifiedEmail': '',
            'BadPassword': i18n('That email or password is not correct.'),
            'Pending': <Spinner />,
            'Error': i18n('Something went wrong. Please try again.'),
            'NotAttempted': ''
        }[attemptStatus];
        return this.props.inline ? this.renderInlineMarkup(email, password, attemptStatus, attemptMessage) : this.renderDefaultMarkup(email, password, attemptStatus, attemptMessage);
    },
    renderDefaultMarkup(email, password, attemptStatus, attemptMessage) {
        return (
            <div className="flow-pane signup-flow">
            <h3><I18N
                        message={'Log in'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Log in</span>;
                        }} /></h3>
            <p><I18N
                        message={'Welcome back! Please enter your password.'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Welcome back! Please enter your password.</span>;
                        }} /></p>
            <div className="field">
                <label><I18N
                            message={'Email address:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Email address:</span>;
                            }} /></label>
                <FormInput type="email" name="email" ref="email"
                        disabled={true}
                        value={email}
                        onChange={Signin.changeHandlerFor('email')}
                        onKeyUp={this.keyAdvance}
                        validation={Valid.emailAddress(email)} />
            </div>
            <div className="field">
                <label><I18N
                            message={'Password'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Password</span>;
                            }} /></label>
                <FormInput type="password" name="password" ref="password"
                        value={password}
                        onKeyUp={this.keyAdvance}
                        onChange={this.passwordChanged}
                        onFocus={() => this.setState({ showPasswordValidation: false })}
                        validation={Valid.gated(this.state.showPasswordValidation, this.passwordValidation(attemptStatus, attemptMessage))} />
            </div>

            <ForgotPassword email={email} />
            {attemptStatus === 'Error' && <p style={{ marginTop: '-2.5em' }}>{attemptMessage}</p>}
            <PaneNavButtons {...this.props}
                    valid={true}
                    atFlowEnd={false}
                    nextButtonText={attemptStatus === 'Pending' ? '...' : i18n('Log in')}
                    goNext={this.login} />
        </div>
        );
    },
    renderInlineMarkup(email, password, attemptStatus, attemptMessage) {
        return (
            <div className="flow-pane signup-flow">
            <div className="create-account">
                <h3><I18N
                            message={'Log in'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Log in</span>;
                            }} /></h3>
                <p><I18N
                            message={'Welcome back! Please enter your password.'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Welcome back! Please enter your password.</span>;
                            }} /></p>
                <div className="field">
                    <label><I18N
                                message={'Email address:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Email address:</span>;
                                }} /></label>
                    <FormInput type="email" name="email" ref="email"
                            value={email}
                            onChange={Signin.changeHandlerFor('email')}
                            onKeyUp={this.keyAdvance}
                            validation={Valid.gated(this.state.showPasswordValidation, Valid.emailAddress(email))} />
                </div>
                <div className="field">
                    <label><I18N
                                message={'Password'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Password</span>;
                                }} /></label>
                    <FormInput type="password" name="password" ref="password"
                            value={password}
                            onKeyUp={this.keyAdvance}
                            onChange={this.passwordChanged}
                            onFocus={() => this.setState({ showPasswordValidation: false })}
                            validation={Valid.gated(this.state.showPasswordValidation, this.passwordValidation(attemptStatus, attemptMessage))} />
                </div>

                <ForgotPassword email={email} />
                {attemptStatus === 'Error' && <p style={{ marginTop: '-2.5em' }}>{attemptMessage}</p>}
                <PaneNavButtons {...this.props}
                        valid={true}
                        atFlowEnd={false}
                        nextButtonText={attemptStatus === 'Pending' ? '...' : i18n('Log in')}
                        prevButtonText={i18n('&lsaquo;&lsaquo; Back to sign up')}
                        goNext={this.login} />
            </div>
            <div className="social-log-in">
                <p><I18N
                            message={'<strong:or>or</strong:or> log in with social media:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span><strong>or</strong> log in with social media:</span>;
                            }} /></p>
                <Engage.Authenticator onLogin={this.janrainLoginSucceeded} onSignup={this.janrainSignupInitiated} />
            </div>
        </div>
        );
    },
    passwordChanged(evt) {
        Signin.changeHandlerFor('password')(evt);
        this.setState({ showPasswordValidation: false });
    },
    passwordValidation(attemptStatus, errorMsg) {
        var ok = attemptStatus !== 'Failure';
        return Valid.allValid([
            Valid.notEmpty(Signin.getField('password')),
            {
                ok,
                errorMsg
            }
        ]);
    },
    keyAdvance: function (evt) {
        if (evt.key === 'Enter') {
            evt.preventDefault();
            this.login();
        }
    },
    componentDidMount: function () {
        this.refs.password.focus();
    }
});
module.exports = LoginPassword;
