'use strict';
var Immutable = require('immutable');
var React = require('react/addons');
var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var Valid = require('abc/validators');
var Wish = require('Wish');
var Worldly = require('Worldly');
var FormInput = require('elements/FormInput');
var If = require('abc/if');
var LocationField = require('elements/LocationField');
var PaneNavButtons = require('elements/PaneFlow').PaneNavButtons;
var Engage = require('./JanrainEngage');
var FieldValidationHelper = require('./FieldValidationHelper');
var {I18N, i18n} = require('I18N');
var SignupStartPane = React.createClass({
    mixins: [
        Worldly,
        FieldValidationHelper,
        Engage.Mixin
    ],
    getDefaultProps: function () {
        return {
            janrain: true,
            inline: false
        };
    },
    getInitialState: function () {
        return { showValidation: false };
    },
    canAdvanceEmail: function () {
        if (this.isValid()) {
            return Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')), {
                Pending: () => false,
                Available: () => true,
                Error: () => false
            });
        } else {
            return false;
        }
    },
    componentDidUpdate() {
        if (this.state.triedToAdvance) {
            Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')), {
                Pending: () => {
                },
                Error: () => this.setState({ triedToAdvance: false }),
                Available: () => this.checkEmailAndProceed()
            });
        }
    },
    checkEmailAndProceed: function () {
        if (this.isValid()) {
            Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')), {
                Pending: () => this.setState({ triedToAdvance: true }),
                Error: () => null,
                Available: exists => {
                    this.props.switchToLoginFlow(exists);
                    this.props.goNext();
                }
            });
        } else {
            this.setState({
                showValidation: true,
                triedToAdvance: false
            });
        }
    },
    render: function () {
        return this.props.inline ? this.renderInlineMarkup() : this.renderDefaultMarkup();
    },
    renderDefaultMarkup: function () {
        var email = Signin.getField('email');
        return (
            <div className="flow-pane signup-flow">
            <div className="sign-in">
                <h1><I18N
                            message={'Sign in with:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Sign in with:</span>;
                            }} /></h1>
                <Engage.Authenticator onLogin={this.janrainLoginSucceeded} onSignup={this.janrainSignupInitiated} />
                <div className="field">
                    <label><I18N
                                message={'Or, use your email address:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Or, use your email address:</span>;
                                }} /></label>
                    <FormInput type="email" name="email" ref="emailAddress"
                            value={email}
                            onChange={Signin.changeHandlerFor('email')}
                            onKeyUp={this.keyAdvance}
                            onFocus={() => this.setState({ showValidation: false })}
                            validation={Valid.gated(this.state.showValidation, Valid.emailAddress(email))} />
                </div>
            </div>
            <PaneNavButtons {...this.props}
                    onInvalid={() => this.setState({ showValidation: true })}
                    valid={this.canAdvanceEmail()}
                    goNext={this.checkEmailAndProceed} />
        </div>
        );
    },
    renderInlineMarkup: function () {
        var email = Signin.getField('email');
        return (
            <div className="flow-pane signup-flow">
            <div className="create-account">
                <div className="email-signin">
                    <label><I18N
                                message={'Enter your email address:'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Enter your email address:</span>;
                                }} /></label>
                    <FormInput type="email" name="email" ref="emailAddress"
                            value={email}
                            onChange={Signin.changeHandlerFor('email')}
                            onKeyUp={this.keyAdvance}
                            onFocus={() => this.setState({ showValidation: false })}
                            validation={Valid.gated(this.state.showValidation, Valid.emailAddress(email))} />
                </div>
                <PaneNavButtons {...this.props}
                        onInvalid={() => this.setState({ showValidation: true })}
                        valid={this.canAdvanceEmail()}
                        goNext={this.checkEmailAndProceed} />
            </div>
            <div className="social-log-in">
                <span><I18N
                            message={'or use social media:'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>or use social media:</span>;
                            }} /></span>
                <Engage.Authenticator onLogin={this.janrainLoginSucceeded} onSignup={this.janrainSignupInitiated} />
                <p className="log-in">
                    <I18N
                            message={'Already have an Idealist account?\xA0\n                        <a:login href="#">\n                            Log in\n                        </a:login>\xA0\n                    to get started.'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return (
                                    <span>Already have an Idealist account?&nbsp;
                        <a href="#" ref="loginLink" onClick={this.forceSwitchToLogin}>
                            Log in
                        </a>&nbsp;
                    to get started.</span>
                                );
                            }} />
                </p>
            </div>
        </div>
        );
    },
    keyAdvance: function (evt) {
        if (evt.key === 'Enter') {
            evt.preventDefault();
            this.checkEmailAndProceed();
        }
    },
    isValid: function () {
        return Valid.emailAddress(Signin.getField('email')).ok;
    },
    focus: function () {
        if (!this.props.inline) {
            this.refs.emailAddress.focus();
        }
    },
    forceSwitchToLogin: function () {
        this.props.switchToLoginFlow(true);
        this.props.goNext();
    }
});
module.exports = SignupStartPane;
