'use strict';
var _ = require('underscore');
var React = require('react/addons');
var classNames = require('classnames');
var App = require('dealers/App');
var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');
var Actions = require('Actions');
var Router = require('Router');
var Wish = require('Wish');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var Link = require('elements/Link');
var PageHeaderAssetSearches = require('./PageHeaderAssetSearches');
var Search = require('./PageHeaderSearch');
var PageHeaderUserMenu = require('./PageHeaderUserMenu');
var SystemMessages = require('./SystemMessages');
var DesktopSubHeader = React.createClass({
    propTypes: { onPostListing: React.PropTypes.func.isRequired },
    render() {
        return (
            <div className="susify full-size">
            <Search.PageHeaderSearch id="searchBox" />
            <div className="site-actions">
                <a className="button post-listing" onClick={this.props.onPostListing} href="#">
                    <I18N
                            message={'Post a listing'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Post a listing</span>;
                            }} />
                </a>
            </div>
        </div>
        );
    }
});
var MobileSubHeader = React.createClass({
    render() {
        return (
            <div className="susify mobile-size">
            <a href="#footer-search"><I18N
                        message={'Search'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Search</span>;
                        }} /></a>
            <a className="post-listing" onClick={this.props.onPostListing} href="#"><I18N
                        message={'Post'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Post</span>;
                        }} /></a>
            <a href="#footer-more"><I18N
                        message={'More'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>More</span>;
                        }} /></a>
        </div>
        );
    }
});
Actions.register('PageHeader/RedirectIfAuthenticated', route => {
    return {
        effect: world => {
            Wish.look(App.currentUser(world), {
                Pending: () => {
                },
                Error: () => {
                },
                Available: user => {
                    user = user.toJS();
                    if (!user.unauthorized) {
                        document.location.href = Router.urlForRoute(route);
                    }
                }
            });
        }
    };
});
var PageHeader = React.createClass({
    mixins: [Worldly],
    getInitialState() {
        return { wantsToPostListing: false };
    },
    onPostListing(event) {
        event.preventDefault();
        this.setState({ wantsToPostListing: true });
    },
    componentWillUpdate(nextProps, nextState) {
        if (!nextState.wantsToPostListing) {
            return;
        }
        var currentUser = App.currentUser(this.world());
        Wish.look(currentUser, {
            Pending: () => {
            },
            Available: user => {
                this.directUserToAddOrgOrCreateContent(user);
                _.defer(() => this.setState({ wantsToPostListing: false }));
            },
            Error: () => {
            }
        });
    },
    directUserToAddOrgOrCreateContent(user) {
        var user = user.toJS();
        var createContent = Router.route('step/track/passthrough', {
            step: 'join-org',
            segment: '-',
            identifier: 'post',
            path: 'create-content'
        });
        var addOrg = Router.route('step/track/passthrough', {
            step: 'join-org',
            segment: '-',
            identifier: 'post',
            path: 'add-org'
        });
        if (user.known_admin && user.unauthorized) {
            LoginSignup.openSignup(this.world(), {
                name: 'PageHeader/RedirectIfAuthenticated',
                params: [createContent]
            });
        } else if (user.org_admin) {
            document.location.href = Router.urlForRoute(createContent);
        } else {
            document.location.href = Router.urlForRoute(addOrg);
        }
    },
    render: function () {
        var headerClasses = NavMenu.slidingClassNames(this.world());
        return (
            <header className={classNames('ido-header', headerClasses)} role="banner">
            <div className="sup-header">
                <div className="susify">
                    <div className="brand">
                        <h1>
                            <span className="site-title hide-text"><I18N
                                        message={'Idealist.org'}
                                        context={this}
                                        args={[]}
                                        fallback={function () {
                                            return <span>Idealist.org</span>;
                                        }} /></span>
                            <a href="/" title="Idealist.org">
                                <img className="idealist-logo" src="/images/logos/idealist.gif" width="180"
                                        alt="Idealist.org" />
                            </a>
                        </h1>
                    </div>
                    <a className="jumplink" href="#main" tabIndex="1"><I18N
                                message={'Jump to content'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Jump to content</span>;
                                }} /></a>
                    <PageHeaderAssetSearches />
                    <PageHeaderUserMenu />
                </div>
            </div>
            <div className="sub-header">
                <DesktopSubHeader onPostListing={this.onPostListing} />
                <MobileSubHeader onPostListing={this.onPostListing} />
            </div>
            <SystemMessages />
        </header>
        );
    }
});
module.exports = PageHeader;
