'use strict';
var _ = require('underscore');
var React = require('react/addons');
var App = require('dealers/App');
var Auth = require('dealers/Auth');
var Client = require('dealers/Client');
var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');
var ClippedNumber = require('elements/ClippedNumber');
var Link = require('elements/Link');
var Look = require('elements/Look');
var Wish = require('Wish');
var If = require('abc/if');
var PopupMenu = require('abc/popup_menu');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var AdministeredAssets = require('./AdministeredAssets');
var PageHeaderUserMenu = React.createClass({
    mixins: [Worldly],
    getInitialState: function () {
        return { showingSignin: false };
    },
    unauthedUserMenu: function () {
        return (
            <ul>
            <li className="login" id="paLogIn">
                <button className="button" onClick={this.openSignup}><I18N
                            message={'Log in'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Log in</span>;
                            }} /></button>
            </li>
            <li className="signUp" id="paSignUp">
                <button className="button" onClick={this.openSignup}><I18N
                            message={'Sign up'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Sign up</span>;
                            }} /></button>
            </li>
        </ul>
        );
    },
    authedUserMenu: function (currentUser) {
        var currentUser = currentUser.toJS();
        var administeredAssets = App.currentAdministeredAssets(this.world());
        var menuTriggerElement = (
            <div id="me" className="current uActions">
            <Link className="me-menu" endpoint='me/view'>
                <span className="short-name">{currentUser.first_name}</span>&nbsp;
                <span className="msgCount">
                    <ClippedNumber value={currentUser.inbox_count} />
                </span>
            </Link>
        </div>
        );
        return (
            <ul className="user-links">
            {!Client.isMobile(this.world()) ? (
                    <li id="currentUser">
                    <PopupMenu dropTab={true} className="me-menu-body menu-items scrolling" trigger={menuTriggerElement}>
                        {}
                        <ul id="meArea" className="user-navlinks">
                            <li id="myProfileLink" className="dropListItem">
                                <Link endpoint='me/view'>
                                    <figure className="avatar-icon">
                                        <img src={currentUser.profile_img_url} alt={currentUser.full_name} className="my-avatar" />
                                        <figcaption className="profile-link"><I18N
                                                    message={'Profile'}
                                                    context={this}
                                                    args={[]}
                                                    fallback={function () {
                                                        return <span>Profile</span>;
                                                    }} /></figcaption>
                                    </figure>
                                </Link>
                            </li>
                            <li id="myInboxLink" className="dropListItem">
                                <Link endpoint='me/messages'>Inbox ({currentUser.inbox_count})</Link>
                            </li>
                            <li id="myDashboardLink" className="dropListItem">
                                <Link endpoint='me/dashboard'><I18N
                                            message={'Dashboard'}
                                            context={this}
                                            args={[]}
                                            fallback={function () {
                                                return <span>Dashboard</span>;
                                            }} /></Link>
                            </li>
                        </ul> {}

                        {}
                        <Look.Quick wish={administeredAssets}>
                            {assets => <AdministeredAssets assets={assets} />}
                        </Look.Quick>{}

                        <span id="logout" className="dropListItem log-out">
                            <Link className="logout" endpoint='logout'><I18N
                                        message={'Log out'}
                                        context={this}
                                        args={[]}
                                        fallback={function () {
                                            return <span>Log out</span>;
                                        }} /></Link>
                        </span>
                    </PopupMenu>
                </li>
                ) : (
                    <a id="nav-toggle" className="me-menu" href="#"
                        onClick={this.toggleMobileNavMenu}>
                    <i className="navicon"></i>
                    <span className="short-name">{currentUser.first_name}</span>
                </a>
                )}
        </ul>
        );
    },
    toggleMobileNavMenu(event) {
        event.preventDefault();
        NavMenu.toggleMobileNavMenu(this.world());
    },
    render: function () {
        var currentUser = App.currentUser(this.world());
        return (
            <nav className="user-nav" role="navigation">
            <Look wish={currentUser}>
                {{
                        Available: function (value) {
                            if (Auth.isUnauthed(this.world())) {
                                return this.unauthedUserMenu(value);
                            }
                            return this.authedUserMenu(value);
                        }.bind(this),
                        Pending: () => null,
                        Error: this.unauthedUserMenu
                    }}
            </Look>
        </nav>
        );
    },
    openSignup: function () {
        LoginSignup.openSignup(this.world());
    }
});
module.exports = PageHeaderUserMenu;
