var React = require('react');
var {Spring, TransitionSpring, presets} = require('react-motion');
var SpringPresets = presets;
var _ = require('underscore');
var Immutable = require('immutable');
var ReactIntl = require('react-intl');
var config = require('config');
var API = require('API');
var App = require('dealers/App');
var Auth = require('dealers/Auth');
var LoginSignup = require('dealers/LoginSignup');
var Router = require('Router');
var Wish = require('Wish');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var LegacyI3APIHeaders = require('legacy/I3APIHeaders');
var Link = require('elements/Link');
var Look = require('elements/Look');
var UserIntake = require('elements/OrgIntake/UserIntake');
var OrgIntakeCore = require('elements/OrgIntake/OrgIntakeCore');
var OrgIntakeForm = require('dealers/OrgIntakeForm');
var Spinner = require('elements/Spinner');
var Org = require('schemas/Org');
var Promo = React.createClass({
    getInitialState() {
        return {
            arrowOpacity: 1,
            jumping: false,
            finalScroll: null
        };
    },
    componentDidMount() {
        this._scrollHandler = this.fadeArrow;
        if (window.addEventListener) {
            window.addEventListener('scroll', this._scrollHandler);
        } else {
            window.attachEvent('onscroll', this._scrollHandler);
        }
        this.fadeArrow();
    },
    componentWillUnmount() {
        if (window.addEventListener) {
            window.removeEventListener('scroll', this._scrollHandler);
        } else {
            window.detachEvent('onscroll', this._scrollHandler);
        }
    },
    promoUrl() {
        var code = 'en_us';
        var url = config.idealist['org_intake.promo.en_us'];
        return url + '?wmode=transparent&rel=0&autohide=1&showinfo=0';
    },
    fadeArrow() {
        var topWindow = window.scrollY;
        var topWindow = topWindow * 1.5;
        var windowHeight = window.innerHeight;
        var position = topWindow / windowHeight;
        position = 1 - position;
        this.setState({ aboveFoldPosition: position });
    },
    arrowJump(event) {
        event.preventDefault();
        var initialTime = Date.now();
        var initialScroll = window.scrollY;
        var node = document.querySelector('#start-org-intake');
        var finalScroll = 0;
        while (node !== document.body) {
            finalScroll += node.offsetTop;
            node = node.offsetParent;
        }
        this.setState({
            jumping: true,
            finalScroll: finalScroll
        });
    },
    render() {
        var hasVideo = true;
        var dailyVisitors = 100000;
        var opportunityCount = 30000;
        return (
            <section className="intro susify">
            <div className="block">
                <div className="message">
                    <h1 className="title"><I18N
                                message={'Share your opportunities.<br />Find the right people.'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Share your opportunities.<br />Find the right people.</span>;
                                }} /></h1>
                    <p><I18N
                                message={'Recruit staff, volunteers, and supporters. Promote your programs and services to the millions who visit us each month.'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span>Recruit staff, volunteers, and supporters. Promote your programs and services to the millions who visit us each month.</span>;
                                }} /></p>
                </div>

                {hasVideo ? (
                        <div className="promo-content">
                    <iframe src={this.promoUrl()} frameBorder="0" allowFullScreen="true"></iframe>
                </div>
                    ) : (
                        <div className="promo-content image-fallback">
                    <img src={this.promoUrl()} />
                </div>
                    )}

                <ul className="stats">
                    <li><I18N
                                message={'<span:stat><ReactIntl.FormattedNumber /></span:stat>daily visitors'}
                                context={this}
                                args={[
                                    ReactIntl,
                                    dailyVisitors
                                ]}
                                fallback={function () {
                                    return <span><span className="stat"><ReactIntl.FormattedNumber value={dailyVisitors} /></span>daily visitors</span>;
                                }} /></li>
                    <li><I18N
                                message={'<span:stat>{this.props.orgCount}</span:stat>organizations'}
                                context={this}
                                args={[]}
                                fallback={function () {
                                    return <span><span className="stat">{this.props.orgCount}</span>organizations</span>;
                                }} /></li>
                    <li><I18N
                                message={'<span:stat><ReactIntl.FormattedNumber /></span:stat>opportunities'}
                                context={this}
                                args={[
                                    ReactIntl,
                                    opportunityCount
                                ]}
                                fallback={function () {
                                    return <span><span className="stat"><ReactIntl.FormattedNumber value={opportunityCount} /></span>opportunities</span>;
                                }} /></li>
                </ul>

                <a id="scroll-arrow" className="arrow-wrap" href="#start-org-intake"
                        style={{ opacity: this.state.aboveFoldPosition }}
                        onClick={this.arrowJump}>
                    <span className="arrow"></span>
                </a>

                {this.state.jumping && (
                        <Spring defaultValue={{ val: window.scrollY }} endValue={{ val: this.state.finalScroll }}>
                    {interpolated => {
                                let nextVal = interpolated.val;
                                if (Math.abs(interpolated.val - this.state.finalScroll) < 1) {
                                    setImmediate(() => this.setState({ jumping: false }));
                                    nextVal = this.state.finalScroll;
                                }
                                window.scrollTo(0, nextVal);
                                return null;
                            }}
                </Spring>
                    )}
            </div>
        </section>
        );
    }
});
var OrgIntakePricing = React.createClass({
    render: function () {
        return (
            <Look wish={this.props.markup}>
            {{
                    Pending: () => (
                        <div key="pending" className="pricing paid">
                <h2><I18N
                                    message={'What kind of opportunities can my org post?'}
                                    context={this}
                                    args={[]}
                                    fallback={function () {
                                        return <span>What kind of opportunities can my org post?</span>;
                                    }} /></h2>
                <p
                                style={{
                                    textAlign: 'center',
                                    margin: 80
                                }}>
                    <Spinner />
                </p>
            </div>
                    ),
                    Error: () => <span>Error</span>,
                    Available: markup => (
                        <div key="available"
                            dangerouslySetInnerHTML={{
                                __html: markup.getIn([
                                    'updates',
                                    'body'
                                ])
                            }} />
                    )
                }}
        </Look>
        );
    }
});
var OrgLogos = React.createClass({
    mixins: [Worldly],
    logos: {
        'en': [
            [
                'World Food Programme',
                '/images/org/org-intake/en/wfp.png'
            ],
            [
                'American Red Cross',
                '/images/org/org-intake/en/american-red-cross.png'
            ],
            [
                'City Year',
                '/images/org/org-intake/en/city-year.png'
            ],
            [
                'Amnesty International',
                '/images/org/org-intake/en/amnesty-international.png'
            ],
            [
                'OXFAM',
                '/images/org/org-intake/en/oxfam.png'
            ],
            [
                'International Federation',
                '/images/org/org-intake/en/international-federation.png'
            ],
            [
                'Sierra Club',
                '/images/org/org-intake/en/sierra-club.png'
            ]
        ],
        'es': [
            [
                'Ashoka Changemakers',
                '/images/org/org-intake/es/ashoka-changemakers.png'
            ],
            [
                'World Food Programme',
                '/images/org/org-intake/en/wfp.png'
            ],
            [
                'Amnesty International',
                '/images/org/org-intake/en/amnesty-international.png'
            ],
            [
                'OXFAM',
                '/images/org/org-intake/en/oxfam.png'
            ],
            [
                'TECHO',
                '/images/org/org-intake/es/techo.png'
            ],
            [
                'International Federation',
                '/images/org/org-intake/en/international-federation.png'
            ],
            [
                'Wingu | Tecnolog\xEDa Sin Fines De Lucro',
                '/images/org/org-intake/es/wingu.png'
            ]
        ],
        'fr': [
            [
                'World Food Programme',
                '/images/org/org-intake/en/wfp.png'
            ],
            [
                'Amnesty International',
                '/images/org/org-intake/en/amnesty-international.png'
            ],
            [
                'OXFAM',
                '/images/org/org-intake/en/oxfam.png'
            ],
            [
                'International Federation',
                '/images/org/org-intake/en/international-federation.png'
            ],
            [
                'Compter sur demain',
                '/images/org/org-intake/fr/compter-sur-demain.png'
            ],
            [
                'American Red Cross',
                '/images/org/org-intake/en/american-red-cross.png'
            ],
            [
                'L\'Arche \xE0 Paris',
                '/images/org/org-intake/fr/larche.png'
            ]
        ]
    },
    render() {
        return (
            <div>
        <section className="org-logos ">
          <h1 className="block-title susify">
              <I18N
                            message={'Join more than {this.props.orgCount} organizations from around the world'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Join more than {this.props.orgCount} organizations from around the world</span>;
                            }} />
          </h1>
          <div className="susify">
            <div className="block">
              <ul>
                  {this.logos[this.world().get('Locale')].map(([
                                    alt,
                                    url
                                ]) => <li key={url}><img src={url} alt={alt} title={alt} /></li>)}
              </ul>
            </div>
          </div>
        </section>
      </div>
        );
    }
});
function OrgCountWish(world) {
    return API.get(world, [
        'Stats',
        'orgCount'
    ], Router.route('api/v3/stats', { orgcount: true }));
}
var OrgIntake = React.createClass({
    mixins: [Worldly],
    statics: {
        initialWishes: world => [OrgCountWish],
        htmlHead: () => {
            return {
                title: i18n('Add Your Org'),
                stylesheet: 'org-intake'
            };
        }
    },
    getEndValue() {
        return Auth.isDefinitelyUnauthed(this.world()) ? {
            intake: {
                top: { val: 0 },
                opacity: { val: 1 }
            }
        } : {};
    },
    willLeave() {
        return {
            top: { val: -this._userIntake.clientHeight },
            opacity: {
                val: 0,
                config: SpringPresets.stiff
            }
        };
    },
    render() {
        var authed = !Auth.isUnauthed(this.world());
        var pendingVerified = Auth.emailAddressPendingVerification(this.world());
        var orgType = OrgIntakeForm.getField('type');
        var address = OrgIntakeForm.getAddress();
        var country = address.get('countryCode') || 'US';
        var locale = this.world().get('Locale');
        var pricingMarkup = API.legacyGet(this.world(), [
            'OrgIntake',
            'Pricing',
            country,
            orgType,
            locale
        ], Router.route('org-pricing-summary', {
            country_code: country,
            org_type: Org.CAPITALIZED_TYPES[orgType],
            language_code: locale
        }));
        var orgCountDisplay = (
            <Look
                wish={this.world().findExistingWish([
                    'Stats',
                    'orgCount'
                ])}>{{
                    Pending: () => <Spinner />,
                    Error: () => <span>Error</span>,
                    Available: wish => {
                        var orgCount = wish.getIn([
                            'data',
                            'orgcount'
                        ]);
                        return <ReactIntl.FormattedNumber value={orgCount} />;
                    }
                }}</Look>
        );
        return (
            <div>
            <Promo orgCount={orgCountDisplay} />

            <section className="how-to-post" id="start-org-intake">
              <h1 className="block-title susify"><I18N
                            message={'How do I post an opportunity on Idealist?'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>How do I post an opportunity on Idealist?</span>;
                            }} /></h1>
              <div className="full-width">
                <ul className="three-steps susify">
                  <li className={!authed && 'current-step step-1'}>
                    <I18N
                                    message={'<h2>1. Create a personal account</h2>\n                    <p>It only takes a second to create your login and password. And it\u2019s free!</p>'}
                                    context={this}
                                    args={[]}
                                    fallback={function () {
                                        return (
                                            <span><h2>1. Create a personal account</h2>
                    <p>It only takes a second to create your login and password. And it’s free!</p></span>
                                        );
                                    }} />
                  </li>
                  <li className={authed && 'current-step step-2'}>
                    <I18N
                                    message={'<h2>2. Add your organization</h2>\n                    <p>You\u2019re doing awesome work. Tell the world! (This is also free.)</p>'}
                                    context={this}
                                    args={[]}
                                    fallback={function () {
                                        return (
                                            <span foo="bar"><h2>2. Add your organization</h2>
                    <p>You’re doing awesome work. Tell the world! (This is also free.)</p></span>
                                        );
                                    }} />
                  </li>
                  <li>
                    <I18N
                                    message={'<h2>3. Post away!</h2>\n                    <p>There are tons of idealists looking to help your cause. Help them find you.</p>'}
                                    context={this}
                                    args={[]}
                                    fallback={function () {
                                        return (
                                            <span><h2>3. Post away!</h2>
                    <p>There are tons of idealists looking to help your cause. Help them find you.</p></span>
                                        );
                                    }} />
                  </li>
                </ul>
              </div>
            </section>

            <TransitionSpring endValue={this.getEndValue} willLeave={this.willLeave}>
                {interpolated => {
                        let translate = `translateY(${ interpolated.intake ? interpolated.intake.top.val : 0 }px)`;
                        return (
                            <div
                                style={{
                                    transform: translate,
                                    'WebkitTransform': translate,
                                    'msTransform': translate,
                                    'OTransform': translate
                                }}>

                        {interpolated.intake && !LoginSignup.isModalSignupOpen(this.world()) && <UserIntake ref={component => this._userIntake = React.findDOMNode(component)} opacity={interpolated.intake.opacity.val} pendingVerified={pendingVerified} />}

                        {}
                        <section className="org-intake">
                          {authed ? (
                                        <h1 className="block-title susify step-2"><I18N
                                                message={'<span:step-2>Step 2: </span:step-2>Add your organization to Idealist'}
                                                context={this}
                                                args={[]}
                                                fallback={function () {
                                                    return <span><span className="step-text">Step 2: </span>Add your organization to Idealist</span>;
                                                }} /></h1>
                                    ) : (
                                        <h1 className="block-title susify what-kind"><I18N
                                                message={'What kind of opportunities can my org post?'}
                                                context={this}
                                                args={[]}
                                                fallback={function () {
                                                    return <span>What kind of opportunities can my org post?</span>;
                                                }} /></h1>
                                    )}
                          <div className="susify">
                            <div className="block" id="org-intake-component">
                                <OrgIntakeCore authed={authed} />
                                <OrgIntakePricing markup={pricingMarkup} />
                            </div>
                          </div>
                        </section>

                        <OrgLogos orgCount={orgCountDisplay} />
                    </div>
                        );
                    }}
            </TransitionSpring>
        </div>
        );
    },
    componentWillMount() {
        OrgIntakeForm.initForm(this);
        OrgIntakeForm.state.observe((success, changedField) => {
            if (changedField.name === 'type' || changedField.name === 'addresses' || changedField === OrgIntakeForm.state) {
                this.forceUpdate();
            }
        });
    },
    componentDidMount() {
        if (window.location.search.indexOf('?activation_action') === 0) {
            OrgIntakeForm.setField('activationActions', [decodeURIComponent(location.search.substring(location.search.indexOf('=') + 1))]);
        }
    },
    componentWillUnmount() {
        this.world().performAction({
            name: 'SignupFlow.componentWillUnmount',
            transaction: ws => ws.deleteIn([
                'Auth',
                'hideEmailVerificationMsg'
            ])
        });
    }
});
module.exports = OrgIntake;
