'use strict';
var config = require('config');
var Immutable = require('immutable');
var React = require('react');
var Assets = require('dealers/Assets');
var InfoPages = require('dealers/InfoPages');
var LoginSignup = require('dealers/LoginSignup');
var If = require('abc/if');
var Router = require('Router');
var Wish = require('Wish');
var Worldly = require('Worldly');
var RedirectsOnLoadingErrors = require('RedirectsOnLoadingErrors');
var Link = require('elements/Link');
var Look = require('elements/Look');
var Spinner = require('elements/Spinner');
var blankInfoAsset = Immutable.fromJS({
    name: String.fromCharCode(160),
    description: ''
});
function infoPage(world) {
    return InfoPages.routedInfoPageWithNavigation(world);
}
function actionsForInfoPage(world) {
    return Wish.seq(infoPage(world), doc => Assets.actionsForAsset(world, doc.get('external_id')));
}
var BookmarkButton = React.createClass({
    render: function () {
        return (
            <button onClick={this.props.onClick}>
            {this.props.action.get('bookmarked') ? 'Un-Bookmark' : 'Bookmark'}
        </button>
        );
    }
});
var PageActions = React.createClass({
    mixins: [Worldly],
    bookmarkAction: function () {
        var url = this.props.actions.get('bookmark').get('url');
        if (url) {
            location.href = url;
        } else {
            LoginSignup.openSignup(this.world());
        }
    },
    render: function () {
        var actions = this.props.actions;
        if (typeof actions === 'undefined') {
            return <ul className="page-actions"><li><button style={{ visibility: 'hidden' }}>&nbsp;</button></li></ul>;
        }
        return (
            <ul className="page-actions">
            <li>
                <BookmarkButton onClick={this.bookmarkAction} action={actions.get('bookmark')} />
            </li>
        </ul>
        );
    }
});
var ViewInfoPage = React.createClass({
    mixins: [
        Worldly,
        RedirectsOnLoadingErrors
    ],
    statics: {
        htmlHead: function () {
            return {
                title: 'Idealist.org - Info',
                stylesheet: 'infocenter'
            };
        },
        initialWishes: function (world) {
            return [actionsForInfoPage];
        }
    },
    infoRootPath: function () {
        return Router.routeForWorld(this.world()).get('parameters').get('path').split('/')[0];
    },
    render: function () {
        var [
            doc,
            isError
        ] = Wish.retriever(infoPage(this.world()), blankInfoAsset);
        var currentUrl = Router.routeForWorld(this.world()).get('path');
        var actions = actionsForInfoPage(this.world());
        var tableOfContents = InfoPages.routedInfoNavigation(this.world());
        return (
            <div className="susify infopage">
            {}
            <section className="info-content">
                <h1>{doc('name')}</h1>
                <If test={isError}>
                    <div className="info-text">Error</div>
                </If>
                <If test={!isError}>
                    <div dangerouslySetInnerHTML={{ __html: doc('description') }} className="info-text"></div>
                </If>
            </section>
            <aside>
                <Look wish={actions}>
                    {{
                            Pending: function () {
                                return <PageActions />;
                            },
                            Available: function (actions) {
                                return <PageActions actions={actions} />;
                            },
                            Error: function () {
                                return <PageActions />;
                            }
                        }}
                </Look>

                <section className="info-tree">
                    <h2>Explore</h2>
                    <Look wish={tableOfContents}>
                        {{
                                Pending: () => <Spinner />,
                                Available: toc => <div dangerouslySetInnerHTML={{ __html: toc }}></div>,
                                Error: () => null
                            }}
                    </Look>
                </section>
                <section className="info-search">
                <h2>Search</h2>
                    <form method="get" action={Router.urlForRoute(Router.route('search'))}>
                        <input type="hidden" name="search_type" value="infopage"
                                className="hidden" />
                        <input type="hidden"
                                name={'search_root_path_' + this.infoRootPath()} value="1"
                                className="hidden" />
                        <input type="text" name="search_user_query" placeholder="Search this info center" />
                        <button>
                            <i className="search-icon"></i>
                        </button>
                    </form>
                </section>
            </aside>
        </div>
        );
    }
});
module.exports = ViewInfoPage;
